import os
import requests
from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter
from urllib.parse import urlparse
from requests.exceptions import RequestException
from requests.exceptions import HTTPError


def download(url: str, output_path: str = None, file_name: str = None):
    """
    Download a file from url
    If output_path is None, the file will be downloaded directly at the current directory
    If file_name is None, the file name from the url will be used
    """

    session = __retry_session(
        retries=3,
        backoff_factor=0.1,
        status_forcelist=[429, 500, 502, 503, 504],
        method_whitelist=["GET"],
    )
    try:
        res = session.get(url=url)
    except RequestException as err:
        raise RequestException(err)
    else:
        if res.status_code != 200:
            raise HTTPError("Invalid URL")

        if file_name == "" or file_name is None:
            # Get the file name from the url
            file_name = __get_file_name_from_url(url)

        if output_path == "" or output_path is None:
            file_path = file_name
        else:
            __create_dir(output_path)
            file_path = output_path + "/" + file_name

        with open(file_path, "wb") as file:
            # Download the file in chunks
            for chunk in res.iter_content(chunk_size=1048576):
                if chunk:
                    file.write(chunk)
        return file_path, file_name
    finally:
        session.close()


def __retry_session(
    retries: int, backoff_factor: float, status_forcelist: any, method_whitelist: any
):
    """Retry session"""

    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
        allowed_methods=method_whitelist,
    )

    adapter = HTTPAdapter(max_retries=retry)
    session = requests.Session()
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    return session


def __get_file_name_from_url(url: str):
    """Get file name from url"""

    path = urlparse(url).path
    return path.split("/")[-1]


def __create_dir(directory: str):
    """Create a directory"""

    try:
        os.makedirs(directory, exist_ok=True)
    except OSError as err:
        raise OSError(err)



class GetChromeDriverError(Exception):
    pass


class VersionUrlError(GetChromeDriverError):
    pass


class UnknownPlatformError(GetChromeDriverError):
    pass


class UnknownVersionError(GetChromeDriverError):
    pass


class DownloadError(GetChromeDriverError):
    pass


class VersionError(GetChromeDriverError):
    pass



import os
import platform as pl
import struct
import subprocess
import xml.etree.ElementTree as ElTree
import zipfile
from urllib.request import urlopen

import requests
from requests.exceptions import HTTPError
from requests.exceptions import RequestException

from get_chrome_driver import downloader, constants
from get_chrome_driver.enums import Platform, Phase, OsPlatform
from get_chrome_driver.exceptions import (
    GetChromeDriverError,
    UnknownPlatformError,
    UnknownVersionError,
    DownloadError,
    VersionError,
    VersionUrlError,
)


class GetChromeDriver:
    def __init__(self, os_platform: OsPlatform = None):
        self.__os_platforms_list = [os_platform for os_platform in OsPlatform]

        if not os_platform:
            if pl.system() == "Windows":
                self.__os_platform = OsPlatform.win
            elif pl.system() == "Linux":
                self.__os_platform = OsPlatform.linux
            elif pl.system() == "Darwin":
                self.__os_platform = OsPlatform.mac
        else:
            if self.__check_if_os_platform_is_valid(os_platform):
                self.__os_platform = os_platform
            else:
                raise UnknownPlatformError(
                    f"Unknown platform, choose a platform from: {str(self.__os_platforms_list)}"
                )

        self.__arch_current = struct.calcsize("P") * 8
        self.__arch_64 = 64

        self.__chromedriver_str = "chromedriver"
        self.__zip_ext = ".zip"

    def stable_version(self) -> str:
        """Return the latest stable version"""

        return self.__latest_version_by_phase(Phase.stable)

    def beta_version(self) -> str:
        """Return the latest beta version"""

        return self.__latest_version_by_phase(Phase.beta)

    def __latest_version_by_phase(self, phase: Phase) -> str:
        """
        Return the latest stable or latest beta version

        :param phase: Stable or beta
        """

        response = requests.get(constants.LAST_KNOWN_GOOD_VERSIONS_URL)
        if not response.ok:
            raise GetChromeDriverError(
                f"Could not get {constants.LAST_KNOWN_GOOD_VERSIONS_URL}"
            )

        try:
            if phase == Phase.stable:
                return response.json()["channels"]["Stable"]["version"]
            if phase == Phase.beta:
                return response.json()["channels"]["Beta"]["version"]
        except KeyError:
            raise UnknownVersionError("Could not find version")

        raise UnknownVersionError("Could not find version")

    def stable_version_url(self) -> str:
        """Return the latest stable version url"""

        return self.version_url(self.__latest_version_by_phase(Phase.stable))

    def beta_version_url(self) -> str:
        """Return the latest beta version url"""

        return self.version_url(self.__latest_version_by_phase(Phase.beta))

    def __version_url_for_platform(
        self,
        new_api_known_good_versions: list,
        version: str,
        platform_64: str,
        platform_32: str = None,
        is_mac: bool = False,
    ) -> str:
        """
        Return the version download url for a platform

        :param new_api_known_good_versions: The latest known good driver versions from the new api
        :param version: Chromedriver version
        :param platform_64: 64bit platform
        :param platform_32: 32bit platform
        """

        url = None

        # New api
        for driver_version in new_api_known_good_versions:
            if driver_version.get("version") == version:
                drivers = driver_version.get("downloads").get("chromedriver")

                # 64
                for driver in drivers or []:
                    if is_mac:
                        _platform_64 = Platform.mac_x64.value
                    else:
                        _platform_64 = platform_64

                    if driver.get("platform") == _platform_64:
                        url = driver.get("url")

                # 32
                if url is None and platform_32 and not is_mac:
                    for driver in drivers or []:
                        if driver.get("platform") == platform_32:
                            url = driver.get("url")

                if url is not None:
                    if self.__check_if_url_is_valid(url):
                        return url

        # Old chromedriver storage
        # 64
        url = f"{constants.CHROMEDRIVER_STORAGE_URL}/{version}/{self.__chromedriver_str}_{platform_64}{self.__zip_ext}"
        if self.__check_if_url_is_valid(url):
            return url

        # 32
        if platform_32:
            url = f"{constants.CHROMEDRIVER_STORAGE_URL}/{version}/{self.__chromedriver_str}_{platform_32}{self.__zip_ext}"
            if self.__check_if_url_is_valid(url):
                return url

        raise VersionUrlError(f"Could not find download url for version {version}")

    def version_url(self, version: str) -> str:
        """
        Return the version download url

        :param version: Chromedriver version
        """

        if not self.__check_if_version_format_is_valid(version):
            raise UnknownVersionError("Invalid version format")

        # Get driver urls from the new api
        response = requests.get(constants.KNOWN_GOOD_VERSIONS_WITH_DOWNLOADS_URL)
        if not response.ok:
            raise GetChromeDriverError(
                f"Could not get {constants.KNOWN_GOOD_VERSIONS_WITH_DOWNLOADS_URL}"
            )
        new_api_known_good_versions = response.json()["versions"]

        if self.__os_platform == OsPlatform.win:
            url = self.__version_url_for_platform(
                new_api_known_good_versions=new_api_known_good_versions,
                version=version,
                platform_32=Platform.win32.value,
                platform_64=Platform.win64.value,
            )

            return url

        elif self.__os_platform == OsPlatform.linux:
            url = self.__version_url_for_platform(
                new_api_known_good_versions=new_api_known_good_versions,
                version=version,
                platform_32=Platform.linux32.value,
                platform_64=Platform.linux64.value,
            )

            return url

        elif self.__os_platform == OsPlatform.mac:
            url = self.__version_url_for_platform(
                new_api_known_good_versions=new_api_known_good_versions,
                version=version,
                platform_64=Platform.mac64.value,
                is_mac=True,
            )

            return url

    def download_stable_version(self, output_path: str = None, extract: bool = False):
        """
        Download the latest stable chromedriver version

        :param output_path: Path to download the driver to
        :param extract: Extract the downloaded driver or not
        """

        version = self.__latest_version_by_phase(Phase.stable)
        self.download_version(version=version, output_path=output_path, extract=extract)

    def download_beta_version(self, output_path: str = None, extract: bool = False):
        """
        Download the latest beta chromedriver version

        :param output_path: Path to download the driver to
        :param extract: Extract the downloaded driver or not
        """

        version = self.__latest_version_by_phase(Phase.beta)
        self.download_version(version=version, output_path=output_path, extract=extract)

    def download_version(
        self, version, output_path: str = None, extract: bool = False
    ) -> str:
        """
        Download a chromedriver version

        :param version: Chromedriver version
        :param output_path: Path to download the driver to
        :param extract: Extract the downloaded driver or not
        """

        if not self.__check_if_version_format_is_valid(version):
            raise UnknownVersionError("Invalid version format")

        if not output_path:
            # On path is None, the driver will be downloaded at e.g. chromedriver/88.0.4324.96/bin/chromedriver.exe
            output_path = self._output_path(version)

        # e.g. if path == 'webdriver/bin', the driver will be downloaded at 'webdriver/bin/chromedriver.exe'
        def download(download_url: str, download_output_path: str) -> str:
            try:
                file_path, file_name = downloader.download(
                    url=download_url, output_path=download_output_path
                )
            except (OSError, HTTPError, RequestException) as err:
                raise DownloadError(err)
            if extract:
                with zipfile.ZipFile(file_path, "r") as zip_ref:
                    zip_ref.extractall(path=download_output_path)
                os.remove(file_path)
                if (
                    self.__os_platform == OsPlatform.linux
                    or self.__os_platform == OsPlatform.mac
                ):
                    os.chmod(f"{download_output_path}/chromedriver", 0o755)
            return download_output_path

        url = self.version_url(version)

        # Download the driver file and return the path of the driver file
        return download(download_url=url, download_output_path=output_path)

    def __check_if_url_is_valid(self, url: str) -> bool:
        """
        Check if url is valid

        :param url: The driver download url
        """

        if requests.head(url).status_code != 200:
            return False

        return True

    def __check_if_version_format_is_valid(self, version: str) -> bool:
        """
        Check if version format is valid

        :param version: Chromedriver version
        """

        split_version = version.split(".")
        for number in split_version:
            if not number.isnumeric():
                return False

        return True

    def __check_if_os_platform_is_valid(self, os_platform: OsPlatform) -> bool:
        """
        Check if platform is valid

        :param os_platform: OS
        """

        if os_platform not in self.__os_platforms_list:
            return False

        return True

    def matching_version(self) -> str:
        """Return a matching ChromeDriver version"""

        all_chromedriver_versions = self.__get_all_chromedriver_versions()
        installed_chrome_version = self.__get_installed_chrome_version()
        for chromedriver_version in reversed(all_chromedriver_versions):
            if ".".join(installed_chrome_version.split(".")[:-1]) == ".".join(
                chromedriver_version.split(".")[:-1]
            ):
                return chromedriver_version

    def auto_download(self, output_path: str = None, extract: bool = False) -> str:
        """
        Download ChromeDriver for the installed Chrome version on machine

        :param output_path: Path to download the driver to
        :param extract: Extract the downloaded driver or not
        """

        version = self.matching_version()
        if version == "" or version is None:
            raise VersionError(
                "Unable to find a ChromeDriver version for the installed Chrome version"
            )

        return self.download_version(version, output_path, extract)

    def install(self):
        """Install ChromeDriver for the installed Chrome version on machine"""

        output_path = self.auto_download(extract=True)
        path = os.path.join(os.path.abspath(os.getcwd()), output_path)
        os.environ["PATH"] += os.pathsep + path

    def __get_all_chromedriver_versions(self) -> list:
        """Return a list with all ChromeDriver versions"""

        key_texts = []
        versions = []

        with urlopen(constants.CHROMEDRIVER_STORAGE_URL) as xml_file:
            tree = ElTree.parse(xml_file)
            root = tree.getroot()

            for root_item in root:
                # Remove namespace
                root_item.tag = root_item.tag.split("}", 1)[1]

            for content in root.findall("Contents"):
                for content_item in content:
                    # Remove namespace
                    content_item.tag = content_item.tag.split("}", 1)[1]

                    key_texts.append(content.find("Key").text)

        for text in key_texts:
            version = ""
            for char in text:
                if char.isnumeric() or char == ".":
                    version += char
                else:
                    break

            if len(version) < 1:
                continue

            versions.append(version)

        return list(dict.fromkeys(versions))

    def __get_installed_chrome_version(self) -> str:
        """Return the installed Chrome version on the machine"""

        if self.__os_platform == OsPlatform.win:
            process = subprocess.Popen(
                [
                    "reg",
                    "query",
                    "HKEY_CURRENT_USER\\SOFTWARE\\Google\\Chrome\\BLBeacon",
                    "/v",
                    "version",
                ],
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL,
                stdin=subprocess.DEVNULL,
            )

            return process.communicate()[0].decode("UTF-8").split()[-1]

        elif self.__os_platform == OsPlatform.linux:
            process = subprocess.Popen(
                ["google-chrome", "--version"],
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL,
                stdin=subprocess.DEVNULL,
            )
            return process.communicate()[0].decode("UTF-8").split()[-1]

        elif self.__os_platform == OsPlatform.mac:
            process = subprocess.Popen(
                [
                    "/Applications/Google Chrome.app/Contents/MacOS/Google Chrome",
                    "--version",
                ],
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL,
                stdin=subprocess.DEVNULL,
            )

            return process.communicate()[0].decode("UTF-8").split()[-1]

    def _output_path(self, version: str) -> str:
        """
        Return the output path

        :param version: Chromedriver version
        """

        return f"chromedriver/{version}/bin"



CHROMIUM_URL = "https://chromedriver.chromium.org"
CHROMEDRIVER_STORAGE_URL = "https://chromedriver.storage.googleapis.com"
LAST_KNOWN_GOOD_VERSIONS_WITH_DOWNLOADS_URL = "https://googlechromelabs.github.io/chrome-for-testing/last-known-good-versions-with-downloads.json"
LAST_KNOWN_GOOD_VERSIONS_URL = "https://googlechromelabs.github.io/chrome-for-testing/last-known-good-versions.json"
KNOWN_GOOD_VERSIONS_WITH_DOWNLOADS_URL = "https://googlechromelabs.github.io/chrome-for-testing/known-good-versions-with-downloads.json"
KNOWN_GOOD_VERSIONS_URL = (
    "https://googlechromelabs.github.io/chrome-for-testing/known-good-versions.json"
)
CSS_SELECTOR_VERSIONS = "ul.n8H08c:nth-child(5)"
LATEST_STABLE_VERSION_STR = "Latest stable release"
LATEST_BETA_VERSION_STR = "Latest beta release"



import typer

from get_chrome_driver import __version__
from get_chrome_driver.enums import Phase, OsPlatform
from get_chrome_driver.exceptions import GetChromeDriverError
from get_chrome_driver.get_driver import GetChromeDriver

app = typer.Typer(name="Get ChromeDriver", add_completion=False)


@app.command()
def main(
    beta_version: bool = typer.Option(
        default=False, help="Print the beta version", show_default=False
    ),
    stable_version: bool = typer.Option(
        default=False, help="Print the stable version", show_default=False
    ),
    latest_urls: bool = typer.Option(
        default=False,
        help="print the beta and stable version download urls for all platforms",
        show_default=False,
    ),
    version_url: str = typer.Option(
        default=None, help="Print the version download url", show_default=False
    ),
    beta_url: bool = typer.Option(
        default=False, help="Print the beta version download url", show_default=False
    ),
    stable_url: bool = typer.Option(
        default=False, help="Print the stable version download url", show_default=False
    ),
    auto_download: bool = typer.Option(
        default=False,
        help="Download a ChromeDriver version for the installed Chrome Version",
        show_default=False,
    ),
    download_beta: bool = typer.Option(
        default=False, help="Download beta version", show_default=False
    ),
    download_stable: bool = typer.Option(
        default=False, help="Download stable version", show_default=False
    ),
    download_version: str = typer.Option(
        default=None, help="Download a specific version", show_default=False
    ),
    extract: bool = typer.Option(
        default=False, help="Extract the compressed driver file", show_default=False
    ),
    version: bool = typer.Option(
        default=False, help="Application version", show_default=False
    ),
):
    """Main"""

    if beta_version:
        __print_latest_version(phase=Phase.beta)

    elif stable_version:
        __print_latest_version(phase=Phase.stable)

    elif latest_urls:
        __print_latest_urls()

    elif version_url:
        __print_version_url(version=version_url)

    elif beta_url:
        __print_latest_url(phase=Phase.beta)

    elif stable_url:
        __print_latest_url(phase=Phase.stable)

    elif auto_download:
        __auto_download(extract=extract)

    elif download_beta:
        __download_latest_version(phase=Phase.beta, extract=extract)

    elif download_stable:
        __download_latest_version(phase=Phase.stable, extract=extract)

    elif download_version:
        __download_version(version=download_version, extract=extract)

    elif version:
        print(f"v{__version__}")


def __print_latest_version(phase: Phase):
    """
    Print latest stable version or latest beta version

    :param phase: Stable or beta
    """

    get_driver = GetChromeDriver()
    error = "No latest version found"
    if phase == Phase.beta:
        try:
            print(get_driver.beta_version())
        except GetChromeDriverError:
            print(error)
    elif phase == Phase.stable:
        try:
            print(get_driver.stable_version())
        except GetChromeDriverError:
            print(error)
    else:
        print(error)


def __print_latest_urls():
    """Print the stable and beta url version for all platforms"""

    get_driver_win = GetChromeDriver(OsPlatform.win)
    get_driver_linux = GetChromeDriver(OsPlatform.linux)
    get_driver_mac = GetChromeDriver(OsPlatform.mac)
    get_drivers = {
        "Windows": get_driver_win,
        "Linux": get_driver_linux,
        "macOS": get_driver_mac,
    }

    result = ""
    for index, (key, value) in enumerate(get_drivers.items()):
        try:
            result += f"Latest beta and stable version for {key}:\n"
            result += f"stable : {value.stable_version_url()}\n"
            result += f"beta   : {value.beta_version_url()}"
            if index < len(get_drivers) - 1:
                result += "\n"
        except GetChromeDriverError:
            continue

    print(result)


def __print_version_url(version: str):
    """
    Print the url of a version

    :param version: Chromedriver version
    """

    get_driver = GetChromeDriver()

    error = "Could not find version url"

    try:
        print(get_driver.version_url(version))
    except GetChromeDriverError:
        print(error)


def __print_latest_url(phase: Phase):
    """
    Print latest stable url or latest beta url

    :param phase: Stable or beta
    """

    get_driver = GetChromeDriver()

    error = "Could not find version url"

    if phase == Phase.beta:
        try:
            print(get_driver.beta_version_url())
        except GetChromeDriverError:
            print(error)
    elif phase == Phase.stable:
        try:
            print(get_driver.stable_version_url())
        except GetChromeDriverError:
            print(error)


def __auto_download(extract: bool):
    """
    Auto download driver

    :param extract: Extract the downloaded driver or not
    """

    get_driver = GetChromeDriver()

    try:
        get_driver.auto_download(extract=extract)
        print("Download finished")
    except GetChromeDriverError:
        print("An error occurred at downloading")


def __download_latest_version(phase: Phase, extract: bool):
    """
    Download the driver for the stable version or beta version

    :param phase: Stable or beta
    :param extract: Extract the downloaded driver or not
    """

    get_driver = GetChromeDriver()

    download_complete = "Download complete"
    beta_error = "Could not download beta version"
    stable_error = "Could not download stable version"

    if phase == Phase.beta:
        try:
            get_driver.download_beta_version(extract=extract)
            print(download_complete)
        except GetChromeDriverError:
            print(beta_error)
    elif phase == Phase.stable:
        try:
            get_driver.download_stable_version(extract=extract)
            print(download_complete)
        except GetChromeDriverError:
            print(stable_error)


def __download_version(version: str, extract: bool):
    """
    Download driver version

    :param version: Chromedriver version
    :param extract: Extract the downloaded driver or not
    """

    get_driver = GetChromeDriver()

    try:
        get_driver.download_version(version=version, extract=extract)
        print("Download finished")
    except GetChromeDriverError:
        print("Could not download version")



from setuptools import setup
from setuptools import find_packages

name = "get-chrome-driver"
version = "1.3.11"

with open("README.md", "r") as fh:
    long_description = fh.read()

requires = ["bs4>=0.0.1", "requests>=2.31.0", "urllib3>=1.26.12", "typer>=0.9.0"]

setup(
    name=name,
    version=version,
    author="Zairon Jacobs",
    author_email="zaironjacobs@gmail.com",
    description="A tool to download and install ChromeDriver.",
    long_description=long_description,
    url="https://github.com/zaironjacobs/get-chrome-driver",
    download_url=f"https://github.com/zaironjacobs/get-chrome-driver/archive/v{version}.tar.gz",
    keywords=["chrome", "chromedriver", "download", "install", "web", "driver", "tool"],
    packages=find_packages(),
    entry_points={
        "console_scripts": [f"{name}=get_chrome_driver.app:app"],
    },
    install_requires=requires,
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
        "Natural Language :: English",
    ],
    python_requires=">=3",
)



import os
import shutil
import subprocess
import platform as pl
from os import path

import pytest
import requests
from bs4 import BeautifulSoup
from decouple import config

from get_chrome_driver import GetChromeDriver
from get_chrome_driver import __version__
from get_chrome_driver import constants

name = "get-chrome-driver"

stable_version = config("STABLE_VERSION")
random_version = config("RANDOM_VERSION")

if pl.system() == "Windows":
    file_name_zipped = "chromedriver_win32.zip"
    file_name = "chromedriver.exe"
    stable_version_url = (
        "https://chromedriver.storage.googleapis.com/"
        + stable_version
        + "/"
        + file_name_zipped
    )
    random_version_url = (
        "https://chromedriver.storage.googleapis.com/"
        + random_version
        + "/"
        + file_name_zipped
    )
elif pl.system() == "Linux":
    file_name_zipped = "chromedriver_linux64.zip"
    file_name = "chromedriver"
    stable_version_url = (
        "https://chromedriver.storage.googleapis.com/"
        + stable_version
        + "/"
        + file_name_zipped
    )
    random_version_url = (
        "https://chromedriver.storage.googleapis.com/"
        + random_version
        + "/"
        + file_name_zipped
    )
elif pl.system() == "Darwin":
    file_name_zipped = "chromedriver_mac64.zip"
    file_name = "chromedriver"
    stable_version_url = (
        "https://chromedriver.storage.googleapis.com/"
        + stable_version
        + "/"
        + file_name_zipped
    )
    random_version_url = (
        "https://chromedriver.storage.googleapis.com/"
        + random_version
        + "/"
        + file_name_zipped
    )

# Change to the current test directory
os.chdir(os.path.dirname(__file__))


class TestApp:
    def test_text_match_latest_stable(self):
        match_found = False

        result = requests.get(constants.CHROMIUM_URL)
        soup = BeautifulSoup(result.content, "html.parser")
        ul = soup.select_one(constants.CSS_SELECTOR_VERSIONS)
        for li in ul:
            text = li.text.replace("\u00A0", " ")
            if (
                text[: len(constants.LATEST_STABLE_VERSION_STR)].lower()
                == constants.LATEST_STABLE_VERSION_STR.lower()
            ):
                match_found = True
                break

        assert match_found is True

    def test_stable_version(self):
        out = subprocess.run(
            args=[name, "--stable-version"],
            universal_newlines=True,
            stdout=subprocess.PIPE,
        )
        actual = out.stdout.split()[0]
        assert stable_version == str(actual)

    def test_random_version_url(self):
        url = random_version_url
        out = subprocess.run(
            args=[name, "--version-url", random_version],
            universal_newlines=True,
            stdout=subprocess.PIPE,
        )
        actual = out.stdout.split()[0]
        assert url, str(actual)

    def test_stable_version_url(self):
        url = stable_version_url
        out = subprocess.run(
            args=[name, "--stable-url"], universal_newlines=True, stdout=subprocess.PIPE
        )
        actual = out.stdout.split()[0]
        assert url == str(actual)

    def test_auto_download_no_extract(self):
        get_driver = GetChromeDriver()
        version = get_driver.matching_version()
        subprocess.run(args=[name, "--auto-download"], stdout=subprocess.PIPE)
        file_path = get_driver._output_path(version) + "/" + file_name_zipped
        result = path.exists(file_path)
        assert result

    def test_auto_download_extract(self):
        get_driver = GetChromeDriver()
        version = get_driver.matching_version()
        subprocess.run(
            args=[name, "--auto-download", "--extract"], stdout=subprocess.PIPE
        )
        file_path_extracted = get_driver._output_path(version) + "/" + file_name
        result = path.exists(file_path_extracted)
        assert result

    def test_auto_download_extract_custom_path(self):
        get_driver = GetChromeDriver()
        get_driver.auto_download(output_path="webdriver/bin/chromedriver", extract=True)
        result = path.exists("webdriver/bin/chromedriver/" + file_name)
        assert result

    def test_download_stable_version_no_extract(self):
        get_driver = GetChromeDriver()
        version = stable_version
        subprocess.run(args=[name, "--download-stable"], stdout=subprocess.PIPE)
        file_path = get_driver._output_path(version) + "/" + file_name_zipped
        result = path.exists(file_path)
        assert result

    def test_download_stable_version_extract(self):
        get_driver = GetChromeDriver()
        version = stable_version
        subprocess.run(
            args=[name, "--download-stable", "--extract"], stdout=subprocess.PIPE
        )
        file_path_extracted = get_driver._output_path(version) + "/" + file_name
        result = path.exists(file_path_extracted)
        assert result

    def test_download_stable_version_extract_custom_path(self):
        get_driver = GetChromeDriver()
        get_driver.download_stable_version(
            output_path="webdriver/bin/chromedriver", extract=True
        )
        result = path.exists("webdriver/bin/chromedriver/" + file_name)
        assert result

    def test_download_random_version_no_extract(self):
        get_driver = GetChromeDriver()
        version = random_version
        subprocess.run(
            args=[name, "--download-version", version], stdout=subprocess.PIPE
        )
        file_path = get_driver._output_path(version) + "/" + file_name_zipped
        result = path.exists(file_path)
        assert result

    def test_download_random_version_extract(self):
        get_driver = GetChromeDriver()
        version = random_version
        subprocess.run(
            args=[name, "--download-version", version, "--extract"],
            stdout=subprocess.PIPE,
        )
        file_path_extracted = get_driver._output_path(version) + "/" + file_name
        result = path.exists(file_path_extracted)
        assert result

    def test_download_random_version_extract_custom_path(self):
        get_driver = GetChromeDriver()
        version = random_version
        get_driver.download_version(
            version, output_path="webdriver/bin/chromedriver", extract=True
        )
        result = path.exists("webdriver/bin/chromedriver/" + file_name)
        assert result

    def test_version(self):
        out = subprocess.run(
            args=[name, "--version"], universal_newlines=True, stdout=subprocess.PIPE
        )
        actual = out.stdout.split()[0]
        assert "v" + __version__ == str(actual)

    @pytest.fixture(scope="function", autouse=True)
    def cleanup(self):
        yield
        try:
            shutil.rmtree("webdriver")
        except (FileNotFoundError, PermissionError):
            pass
        try:
            shutil.rmtree("chromedriver")
        except (FileNotFoundError, PermissionError):
            pass
